package com.ekingwin.eit.web;


import com.ekingwin.eit.service.IEwoPhaseRocketMqService;
import com.ekingwin.eit.service.ITwoPhaseRocketMqService;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.producer.SendResult;
import org.apache.rocketmq.spring.core.RocketMQTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author zugs
 */
@RestController
@RequestMapping("/eitphaserocketmq")
@EnableScheduling
@Api(tags = "测试接口")
@Slf4j
public class EitPhaseRocketMqController {
    //    private static Logger logger = LoggerFactory.getLogger(ProductPhaseRocketMq.class);
    @Resource
    private IEwoPhaseRocketMqService ewoPhaseRocketMqService;
    @Autowired
    private ITwoPhaseRocketMqService twoPhaseRocketMqService;
    @Autowired
    private RocketMQTemplate         rocketMQTemplate;


    @Value("${eit-phase-topic}")
    private String eitPhaseTopic;
    @Value("${ewo-phase-tag}")
    private String  ewoPhaseTag;
    @Value("${two-phase-tag}")
    private String twoPhaseTag;

    @RequestMapping(value = "/testJob",produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ApiOperation("定时处理")
    public void testJob() {
        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        System.out.println(sdf.format(new Date())+"发送成功==========");
        List<Map<String,Object>> EwoAllDatas = ewoPhaseRocketMqService.getEwoAllData();
        List<Map<String,Object>> TwoAllDatas = twoPhaseRocketMqService.getTwoAllData();
        this.sendMqMessage(EwoAllDatas,TwoAllDatas);
        System.out.println(sdf.format(new Date())+"结束==========");
    }

    @RequestMapping(value = "/scheduledJob",produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @Scheduled(cron="0 30 5 * * ?")//定为每天5:30更新
    @ApiOperation("每天定时处理EIT跟PCRS系统MQ接口")
    public void scheduledJob() {
        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        System.out.println(sdf.format(new Date())+"发送开始==========");
        List<Map<String,Object>> EwoAllDatas = ewoPhaseRocketMqService.getEwoAllData();
        List<Map<String,Object>> TwoAllDatas = twoPhaseRocketMqService.getTwoAllData();
        this.sendMqMessage(EwoAllDatas,TwoAllDatas);
        System.out.println(sdf.format(new Date())+"发送结束==========");
    }

//    @RequestMapping(value = "/testJndi",produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
//    @Scheduled(cron="0 */1 * * * ?")//定为每天5:30更新
//    @ApiOperation("测试有没有走")
//    public void testJndi() {
//        System.out.println("测试开始==========");
//        ewoPhaseRocketMqService.getTest();
//        System.out.println("测试结束==========");
//    }


    public void productPhaseMessage(String topic, String tag, Object obj){
        String key = "KEY_" + UUID.randomUUID();
        String payload =null;
        try {
            payload = this.getJsonString(obj);
            System.out.println("参数===="+payload);
            SendResult sendResult = rocketMQTemplate.syncSend(StringUtils.hasText(tag) ?(topic + ":" + tag) : topic,
                    MessageBuilder.withPayload(payload).setHeader("PROPERTY_KEYS", key).build());
            System.out.println("tag===="+tag);
            System.out.println("SendStatus===="+sendResult.getSendStatus());
            System.out.println("MsgId===="+sendResult.getMsgId());
            System.out.println("成功=======" );
        } catch (Exception e) {
            System.out.println("失败===");
            System.out.println("Produce {}:{} failed: {}topic=="+topic+"tag"==tag+"错误消息=="+ e.getMessage());
            System.out.println("失败单子Payload: {}"+ payload);
            System.out.println("失败==="+e);

        }
    }

    private String getJsonString(Object obj) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        return mapper.writeValueAsString(obj);
    }


    public void sendMqMessage( List<Map<String,Object>> EwoAllDatas, List<Map<String,Object>> TwoAllDatas){
        System.out.println("ewo数量==="+EwoAllDatas.size());
        System.out.println("two数量==="+TwoAllDatas.size());
        List<Map<String,Object>> oneData=null;
        for ( Map<String,Object> ewoAllData:EwoAllDatas) {
            oneData=new ArrayList<Map<String,Object>>();
            oneData.add(ewoAllData);
            this.productPhaseMessage(eitPhaseTopic,ewoPhaseTag,oneData);
        }
        for ( Map<String,Object> TwoAllData:TwoAllDatas) {
            oneData=new ArrayList<Map<String,Object>>();
            oneData.add(TwoAllData);
            this.productPhaseMessage(eitPhaseTopic,twoPhaseTag,oneData);

        }
    }
}
